Package: kerasnip
Type: Package
Title: A Bridge Between 'keras' and 'tidymodels'
Version: 0.1.2
Authors@R: 
    person(
        "David",
        "Díaz",
        role = c("aut", "cph", "cre"),
        email = "daviddrsch@gmail.com",
        comment = c(ORCID = "0000-0002-0927-9795")
    )
Description: Provides a seamless bridge between 'keras' and the 'tidymodels'
    frameworks. It allows for the dynamic creation of 'parsnip' model
    specifications for 'keras' models.
Depends: R (>= 4.1.0)
Encoding: UTF-8
License: MIT + file LICENSE
URL: https://davidrsch.github.io/kerasnip/,
        https://github.com/davidrsch/kerasnip
BugReports: https://github.com/davidrsch/kerasnip/issues
RoxygenNote: 7.3.3
Imports: abind, generics, parsnip (>= 1.0.0), rlang, keras3, tibble,
        purrr, dplyr, cli, recipes, reticulate, lobstr
Config/testthat/edition: 3
Suggests: testthat (>= 3.0.0), bundle, butcher, modeldata, tidymodels,
        finetune, tune, dials, workflows, rsample, knitr, lme4,
        rmarkdown, future, ggplot2, mgcv, probably
VignetteBuilder: knitr
NeedsCompilation: no
Packaged: 2026-05-02 18:54:41 UTC; david
Author: David Díaz [aut, cph, cre] (ORCID:
    <https://orcid.org/0000-0002-0927-9795>)
Maintainer: David Díaz <daviddrsch@gmail.com>
Repository: CRAN
Date/Publication: 2026-05-03 00:30:03 UTC
Built: R 4.6.0; ; 2026-05-03 04:37:35 UTC; unix
