# Generated by roxygen2: do not edit by hand

S3method(bake,step_collapse)
S3method(butcher::axe_call,kerasnip_model_fit)
S3method(butcher::axe_ctrl,kerasnip_model_fit)
S3method(butcher::axe_data,kerasnip_model_fit)
S3method(butcher::axe_env,kerasnip_model_fit)
S3method(butcher::axe_fitted,kerasnip_model_fit)
S3method(generics::fit,kerasnip_spec)
S3method(generics::fit_xy,kerasnip_spec)
S3method(glance,kerasnip_model_fit)
S3method(parsnip::set_args,kerasnip_spec)
S3method(parsnip::set_engine,kerasnip_spec)
S3method(prep,step_collapse)
S3method(print,step_collapse)
S3method(required_pkgs,step_collapse)
S3method(stats::predict,kerasnip_model_fit)
S3method(tidy,kerasnip_model_fit)
S3method(tidy,step_collapse)
export(compile_keras_grid)
export(create_keras_functional_spec)
export(create_keras_sequential_spec)
export(extract_keras_history)
export(extract_keras_model)
export(extract_valid_grid)
export(generic_functional_fit)
export(generic_sequential_fit)
export(get_model_env)
export(inform_errors)
export(inp_spec)
export(keras_evaluate)
export(keras_losses)
export(keras_metrics)
export(keras_optimizers)
export(loss_function_keras)
export(model_exists)
export(optimizer_function)
export(process_x_functional)
export(process_x_sequential)
export(process_y_functional)
export(process_y_sequential)
export(register_keras_loss)
export(register_keras_metric)
export(register_keras_optimizer)
export(remove_keras_spec)
export(step_collapse)
importFrom(cli,cli_alert_danger)
importFrom(cli,cli_alert_info)
importFrom(cli,cli_alert_success)
importFrom(cli,cli_bullets)
importFrom(cli,cli_code)
importFrom(cli,cli_h1)
importFrom(cli,cli_h2)
importFrom(cli,cli_text)
importFrom(dplyr,bind_rows)
importFrom(dplyr,filter)
importFrom(dplyr,select)
importFrom(generics,fit)
importFrom(generics,fit_xy)
importFrom(generics,glance)
importFrom(generics,required_pkgs)
importFrom(generics,tidy)
importFrom(keras3,to_categorical)
importFrom(lobstr,obj_size)
importFrom(parsnip,set_args)
importFrom(parsnip,set_engine)
importFrom(parsnip,update_dot_check)
importFrom(recipes,bake)
importFrom(recipes,is_trained)
importFrom(recipes,prep)
importFrom(recipes,sel2char)
importFrom(reticulate,py_require)
importFrom(rlang,arg_match)
importFrom(rlang,dots_list)
importFrom(rlang,enquos)
importFrom(rlang,env_poke)
importFrom(stats,predict)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(utils,tail)
