# print method works

    Code
      print(rec$steps[[1]])
    Output
      Collapse predictors into list-column (unprepped)

---

    Code
      print(prepped_rec$steps[[1]])
    Output
      Collapse predictors into list-column: x1, x2  ->  predictor_matrix 

# print method works for prepped recipe

    Code
      print(rec$steps[[1]])
    Output
      Collapse predictors into list-column: x1, x2  ->  predictor_matrix 

