% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{get_model_env}
\alias{get_model_env}
\title{Get Parsnip's Model Environment}
\usage{
get_model_env()
}
\value{
The \code{parsnip} model environment.
}
\description{
This is an internal helper function to retrieve the environment where
\code{parsnip} stores its model definitions. It is used to dynamically
interact with the \code{parsnip} infrastructure.
}
\examples{
\donttest{
model_env <- kerasnip::get_model_env()
}
}
\keyword{internal}
