% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dials.R
\name{optimizer_function}
\alias{optimizer_function}
\title{Dials Parameter for Keras Optimizers}
\usage{
optimizer_function(values = NULL)
}
\arguments{
\item{values}{A character vector of possible optimizers. Defaults to all
known optimizers (keras defaults + custom registered).}
}
\value{
A \code{dials} parameter object for Keras optimizers.
}
\description{
Dials Parameter for Keras Optimizers
}
\keyword{internal}
