% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/dials.R
\name{loss_function_keras}
\alias{loss_function_keras}
\title{Dials Parameter for Keras Loss Functions}
\usage{
loss_function_keras(values = NULL)
}
\arguments{
\item{values}{A character vector of possible loss functions. Defaults to all
known losses (keras defaults + custom registered).}
}
\value{
A \code{dials} parameter object for Keras loss.
}
\description{
Dials Parameter for Keras Loss Functions
}
\keyword{internal}
