% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/keras_tools.R
\name{glance.kerasnip_model_fit}
\alias{glance.kerasnip_model_fit}
\title{Glance at a Fitted Kerasnip Model}
\usage{
\method{glance}{kerasnip_model_fit}(x, ...)
}
\arguments{
\item{x}{A \code{kerasnip_model_fit} object.}

\item{...}{Not used.}
}
\value{
A one-row tibble with one column per compiled metric. Returns an
empty tibble if training history has been stripped (e.g. by butcher).
}
\description{
Returns a one-row tibble of summary statistics from the final training epoch:
every metric the model was compiled with (e.g. \code{loss}, \code{accuracy}).
}
\keyword{internal}
